# -*- coding: utf-8 -*-
from odoo import api, models, tools, fields, _
from odoo.exceptions import UserError
from odoo.addons.saas_worker.util import email_on_exception

from markupsafe import Markup

import logging
_logger = logging.getLogger(__name__)


# List build based on the most frequent email domain in lead table
# from odoo.com database
# See https://gist.github.com/odony/df23ff9e8cb0caa4f586fd0980c43fee
# If typos appear in the most frequent domains, we want to fix it
COMMON_ESP_DOMAIN_TYPOS = {
    'hku.hku': 'hku.hk',
    'hotmai.com': 'hotmail.com',
    'gamil.com': 'gmail.com',
    'gmai.com': 'gmail.com',
    'gmail': 'gmail.com',
    'gmail.co': 'gmail.com',
    'gmail.con': 'gmail.com',
    'gmail.fr': 'gmail.com',
    'gmail.om': 'gmail.com',
    'gmailcom': 'gmail.com',
    'gmaill.com': 'gmail.com',
    'gmal.com': 'gmail.com',
    'gmial.com': 'gmail.com',
    'gmil.com': 'gmail.com',
}


def correct_domain_typo(email):
    """ Try to find email domain and fix the typo if the domain is in the common typo list """
    email_normalized = tools.email_normalize(email)
    if not email_normalized:
        return email

    domain = email_normalized.rsplit("@", maxsplit=1)[-1]
    domain_corrected = COMMON_ESP_DOMAIN_TYPOS.get(domain, domain)
    return email.replace('@{}'.format(domain), '@{}'.format(domain_corrected))


class CrmLead(models.Model):
    _inherit = 'crm.lead'
    _check_company_auto = False

    allow_send_scaleup = fields.Boolean(compute='_allow_send_scaleup')
    date_open = fields.Datetime(tracking=True)

    def _allow_send_scaleup(self):
        edu_crm_teams = self.env["crm.team"].browse([
            team_id
            for team_id, team_xmlids in self.team_id._get_external_ids().items()
            if any("openerp_crm.crm_team_edu" in xmlid for xmlid in team_xmlids)
        ])
        for lead in self:
            lead.allow_send_scaleup = (
                lead.team_id in edu_crm_teams or 'scaleup' in lead.name.lower()
            )

    @api.depends('partner_id.email')
    def _compute_email_from(self):
        super()._compute_email_from()
        for lead in self:
            lead.email_from = correct_domain_typo(lead.email_from)

    def _inverse_email_from(self):
        for lead in self:
            lead.email_from = correct_domain_typo(lead.email_from)
        super()._inverse_email_from()

    @api.model
    def message_new(self, msg, custom_values):
        lead = super(CrmLead, self).message_new(msg, custom_values=custom_values)
        ref = lead.partner_id and lead.partner_id.subscription_oauth_scope or "answer_mail"
        plan = lead.partner_id and lead.partner_id.plan or "unknown_plan"

        desc = "Ref: %s\n" % ref
        desc += "Question: %s\n" % plan
        desc += lead.description or ''
        lead.description = desc
        return lead

    def partner_interested(self, comment=False):
        # since the text input is replaced by a textarea (see oe_portal_my_lead template),
        # line breaks should be displayed in the chatter.
        return super(CrmLead, self).partner_interested(comment=tools.plaintext2html(comment))

    def action_prepare_picking_scaleup(self):
        products_scaleup = {
            'fr_FR': 'openerp_delivery.product_business_game_product_template_fr',
            'nl_NL': 'openerp_delivery.product_business_game_product_template_nl',
            'es_ES': 'openerp_delivery.product_business_game_product_template_es',
            'en_US': 'openerp_delivery.product_business_game_product_template',
        }
        try:
            product_scaleup_template = products_scaleup[self.lang_id.code]
        except KeyError:
            raise UserError(_('No Scale-Up product available for language: %s (%s)') % (
                self.lang_id.display_name, self.lang_id.code,
            ))
        product = self.env.ref(product_scaleup_template).product_variant_id

        if not product:
            raise ValueError('ScaleUp Product not found')

        customerloc, location_id = self.env['stock.warehouse']._get_partner_locations()

        # public function -> not sudo = restricted to employees
        picking = self.env['stock.picking'].create({
            'partner_id': self.partner_id.id,
            'picking_type_id': self.env.ref('stock.picking_type_out').id,
            'location_id': location_id.id,
            'location_dest_id': customerloc.id,
            'origin': 'LEAD/SCUTEACH/%s' % self.id,
        })

        self.env['stock.move'].create({
            'name': product.name,
            'picking_id': picking.id,
            'product_id': product.id,
            'product_uom': product.uom_id.id,
            'product_uom_qty': 1.0,
            'location_id': picking.location_id.id,
            'location_dest_id': picking.location_dest_id.id,
        })
        self.message_post(body=Markup("Picking created: <a href='#' data-oe-model='stock.picking' data-oe-id='%s'>%s</a> by %s" % (picking.id, picking.name, self.env.user.name)))

        return {
            'name': 'ScaleUp Picking',
            'view_mode': 'form',
            'views': [[False, "form"]],
            'res_model': 'stock.picking',
            'type': 'ir.actions.act_window',
            'nodestroy': True,
            'target': 'self',
            'res_id': picking.id,
            'context': "{}",
        }

    def convert_opportunity(self, partner, user_ids=False, team_id=False):
        res = super().convert_opportunity(partner, user_ids=user_ids, team_id=team_id)
        try:
            self.env.ref("openerp_crm.action_send_whatsapp_message").with_context(
                active_model='crm.lead',
                active_ids=self.ids
            ).run()
        except Exception:
            _logger.exception("Error while sending whatsapp message at lead conversion")
        return res


class CrmLead2opportunityPartner(models.TransientModel):
    _inherit = 'crm.lead2opportunity.partner'

    def action_apply_no_redirect(self):
        super(CrmLead2opportunityPartner, self).action_apply()
        # Overridden to avoid opening the lead view, in order to make the job easier
        # for internal users who convert dozens of leads per day one by one and can't afford to
        # lose their filtered lists after converting each lead.
        return {'type': 'ir.actions.act_window_close'}


class CrmTeam(models.Model):
    _inherit = 'crm.team'

    @email_on_exception()
    def _cron_assign_leads(self, work_days=None):
        return super()._cron_assign_leads(work_days=work_days)

    @api.model
    def get_edu_team(self, by_country):
        default_team = 'openerp_crm.crm_team_edu_be'
        return self.env.ref(default_team)
